{% extends "admin/change_list.html" %}
{% block breadcrumbs %}{% include breadcrumb_template %}{% endblock %}
{% block extrastyle %} {# Fixes a bug in django CMS 4.1rc2 - can be deleted as soon as 4.1 is released #}
    <style>
        span.cms-empty-action { display: none !important; }
    </style>
    {{ block.super }}
{% endblock %}
