# Generated by Django 4.2 on 2023-05-14 10:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_versioning', '0016_alter_version_content_type'),
        ('djangocms_versioning', '0016_auto_20230505_0934'),
    ]

    operations = [
    ]
