{% extends "admin/base_site.html" %}
{% load i18n admin_urls static %}
{% block title %}{% trans "Discard Confirmation" %}{% endblock %}

{% block extrahead %}
    {{ block.super }}
    {{ media }}
    <script type="text/javascript" src="{% static 'admin/js/cancel.js' %}"></script>
{% endblock %}

{% block breadcrumbs %}{% endblock %}
{% block bodyclass %}{{ block.super }} app-{{ opts.app_label }} model-{{ opts.model_name }} delete-confirmation{% endblock %}

{% block content %}
<p>{% translate "Are you sure you want to discard following version?" %}</p>
<h3>{{ object_name }}</h3>
<h4>{% blocktrans %}Version number: {{ version_number }}{% endblocktrans %}</h4>
<form action="" method="POST">
    {% csrf_token %}
    <input  class="button confirm-link js-versioning-keep-sideframe"
            type="submit"
            name="discard"
            value="{% translate "Yes, I'm sure" %}">
    <a href="{{ back_url }}">
        <input type="button"
               class="button js-versioning-keep-sideframe"
               value="{% translate "No, take me back" %}">
    </a>
</form>
{% endblock %}
